#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import math
import copy
import numpy as np

from redcp import * # import data types

from bresenham import bresenham_line

P_FREE = 0.1
P_OCC = 0.9
P_MIN = 0.01

class Mapper:

    def __init__(self):
        pass

    def update_free(self, p_mi):
        """method to calculate the Bayesian update of the free cell with the current occupancy probability value P_mi
        Args:
            p_mi: float64 - current probability of the cell being occupied
        Returns:
            p_mi: float64 - updated probability of the cell being occupied
        """
        p = p_mi * P_FREE
        p_mi = p / (2 * p - P_FREE - p_mi + 1)
        if p_mi < P_MIN: p_min = P_MIN
        if p_mi > (1 - P_MIN): p_mi = (1 - P_MIN)
        return p_mi

    def update_occupied(self, p_mi):
        """method to calculate the Bayesian update of the occupied cell with the current occupancy probability value P_mi
        Args:
            p_mi: float64 - current probability of the cell being occupied
        Returns:
            p_mi: float64 - updated probability of the cell being occupied
        """
        p = p_mi * P_OCC
        p_mi = p / (2 * p - P_OCC - p_mi + 1)
        if p_mi < P_MIN: p_min = P_MIN
        if p_mi > (1 - P_MIN): p_mi = (1 - P_MIN)
        return p_mi

    def fuse_laser_scan(self, grid_map: OccupancyGrid, laser_scan: LaserScan, odometry: Odometry) -> OccupancyGrid:
        """ Method to fuse the laser scan data sampled by the robot with a given 
            odometry into the probabilistic occupancy grid map
        Args:
            grid_map: OccupancyGrid - gridmap to fuse te laser scan to
            laser_scan: LaserScan - laser scan perceived by the robot
            odometry: Odometry - perceived odometry of the robot
        Returns:
            grid_map_update: OccupancyGrid - gridmap updated with the laser scan data
        """
 
        if grid_map is None: ## check if input params are valid
            return None
            
        grid_map_update = copy.deepcopy(grid_map)
        # if laser_scan is None or odometry is None or grid_map.data is None:
        if laser_scan is None or odometry is None:
            return grid_map_update
      
        ##TODO:[t1d-map] fuse the correctly aligned laser data into the probabilistic occupancy grid map

        return grid_map_update
